' *******************************************************************************************************
' *                                                                                                     *
' *     AiChip_JVM_Demo_001.spin                                                                        *
' *                                                                                                     *
' *******************************************************************************************************

PRI Version
  return String("AiChip_JVM_Demo_001")

CON
{
  
PLEASE NOTE : If modifying and re-releasing this code, please change the prefix of the source file from
              AiChip so as to avoid any confusion as to the origin of the source and update all internal
              references as appropriate. If you wish to credit the orginal source, anything  along the
              lines of, "Based on original AiChip_Jvm_XXX from AiChip Industries", is acceptable.

              AiChip is a trademark of AiChip Industries.

}
CON

' .-----------------------------------------------------------------------------------------------------.
' |     Board Configuration                                                                             |
' `-----------------------------------------------------------------------------------------------------'

  _CLKMODE      = XTAL1 + PLL16x                        ' 80MHz System Clock Speed
  _XINFREQ      = 5_000_000

OBJ

  tv            : "TV_Text"                             ' TV Text Display used for debugging
  
  jem           : "AiChip_JVM_JemFile_001.spin"         ' JEM Bytecode to execute ( jem.bin )
  jvm           : "AiChip_JVM_JvmPasm_001.spin"         ' JVM Bytecode Interpreter in PASM
  
CON

' .-----------------------------------------------------------------------------------------------------.
' |     Hardware Configuration                                                                          |
' `-----------------------------------------------------------------------------------------------------'

  TV_PIN        = 12                                    ' Base TV Pin

VAR

  long stack[1024]                                      ' Stack area to use 

  ' These can be renamed but must stay in this order
  ' and must currently all be longs ...
  
  long pc, sp, bp, jemcode, operand, lhs, rhs           ' Interface to Aichip_JVM_Pasm
  
PUB Main

' .-----------------------------------------------------------------------------------------------------.
' |     Initialise Debugging etc                                                                        |
' `-----------------------------------------------------------------------------------------------------'

  tv.Start( TV_PIN )
  tv.Str( Version )

' **************************
' *  ADD YOUR THINGS HERE  *
' **************************

' .-----------------------------------------------------------------------------------------------------.
' |     Initialise JVM                                                                                  |
' `-----------------------------------------------------------------------------------------------------'

' Select one and only one of the following ways to start the JVM. The jvm#XXXX determines what processing
' the JVM will do of its own accord. If NONE it simply gets the the requested opcode/operand and passes
' control back to the Spin handler here. Otherwise it will attempt to handle the opcode/operand by itself
' using its Spin or PASM code as allowed and only pass control back to the Spin handler here if it cannot
' deal with the opcode/operand itself. 

  jvm.Start( jvm#NONE,           jem.ByteCodeStart, @pc, @stack ) ' <-- Currently recommended choice
' jvm.Start( jvm#SPIN_THEN_PASM, jem.ByteCodeStart, @pc, @stack )
' jvm.Start( jvm#PASM_THEN_SPIN, jem.ByteCodeStart, @pc, @stack )
' jvm.Start( jvm#SPIN_ONLY,      jem.ByteCodeStart, @pc, @stack )
' jvm.Start( jvm#PASM_ONLY,      jem.ByteCodeStart, @pc, @stack )

  if sp == 0
    ' Failed to call a jvm.Start trap ...
    ' **************************
    ' *  ADD YOUR THINGS HERE  *
    ' **************************

' .-----------------------------------------------------------------------------------------------------.
' |     Main JVM Loop                                                                                   |
' `-----------------------------------------------------------------------------------------------------'

' At this point ...
'
' pc holds the absolute hub address of the first byte of the JEM bytecode
' sp holds the absolute hub address @stack-4
' bp holds the absolute hub address @stack-4
'
' The stack is pre-increment to push, post-decrement to pop, ie sp always point to TOS
'
' The base pointer (bp) points to the first local variable on stack when a method is executing. It will
' be a fixed offset above the stack frame information needed to do the return so a separate frame pointer
' is not required. The base pointer is the address to which ILOAD, ISTORE and IRET are relative to, so
' 'ILOAD 0' is the same as "Push( long[bp+(0*4)] )", "ISTORE 1" is "long[bp+(1*4)] := Pop". 
'  
' Initialise, adjust pc, sp and bp as necessary, but when done make sure that pc holds the absolute
' address of a JEM opcode in hub memory, the first to be executed.

' **************************
' *  ADD YOUR THINGS HERE  *
' **************************

  repeat 

    ' At this point a JEM bytecode will have been extracted from the hub memory ...
    '
    ' The 'jemcode' variable will hold the byte value of the JEM bytecode
    '
    ' The 'operand' variable will hold one of the following depending upon what operand(s) the bytecode
    ' has -
    '
    '    an undefined value if the opcode has no operand
    '
    '    a positive value (0..255) if the operand is a byte value ( signed or unsigned )
    '
    '    a positive value (0..65535) if the operand is a 16-bit unsigned word
    '
    '    a signed value ( -32768..32767 ) if the operand is a 16-bit signed word
    '
    '    a 16-bit value of "first << 8 | second" if the operand is two 8-bit values
    '
    ' lhs is a positive alue (0..255) of either the first or only byte operand or the MSB of a 16-bit
    ' word operand.
    '
    ' rhs is a positive alue (0..255) of either the second byte operand or the LSB of a 16-bit word
    ' operand or undefined.
    '
    ' The 'pc' will be pointing to the next JEM bytecode having skipped any operands related to the
    ' bytecode.

    ' Handle the JEM bytecode using the 'jemcode', 'operand', 'lhs' and 'rhs' variables, and by pushing
    ' to and pulling from the stack as necessary. For example ...

    case jemcode

      jem#Jem_NOP:              ' Do nothing

      jem#Jem_ILOAD:            Push( long[ bp + (operand*4) ] )

      jem#Jem_IF_ICMPGT:        rhs := Pop
                                lhs := Pop
                                If lhs > rhs
                                  pc := pc + operand
                                
      jem#Jem_IADD:             rhs := Pop
                                lhs := Pop     
                                lhs := lhs + rhs
                                Push( lhs )

      ' **************************
      ' *  ADD YOUR THINGS HERE  *
      ' **************************

PRI Push( aLong )

  long[ ++sp ] := aLong

PRI Pop

  return long[ sp-- ]
                                    